
#include "pixelGameEngine.h"
#include "main.h"
using namespace olc;

extern FaceBall* game;

void Editor::Update(float fElapsedTime){
	vf2d center = { -(float)MAP_SIZE.x / 2 * GRID_SIZE.x + game->ScreenWidth()/2 , -(float)MAP_SIZE.y / 2 * GRID_SIZE.y + game->ScreenHeight()/2 };
	if (game->GetMouseWheel() != 0) {
		if (game->GetMouseWheel() > 0) {
			if (selectedEnemy < 63) {
				selectedEnemy=EnemyID(selectedEnemy+1);
			}
			else {
				selectedEnemy = EnemyID::NONE;
			}
		}
		else {
			if (selectedEnemy > 0) {
				selectedEnemy = EnemyID(selectedEnemy - 1);
			}
			else {
				selectedEnemy = AREA_MAP;
			}
		}
	}
	for (int y = 0; y < MAP_SIZE.y; y++) {
		for (int x = 0; x < MAP_SIZE.x; x++) {
			Tile&t = map[y][x];
			vf2d squarePos = vf2d{ (float)x,(float)y }*GRID_SIZE + center;
			if (game->GetMouseX() >= squarePos.x && game->GetMouseX() <= squarePos.x + GRID_SIZE.x &&
				game->GetMouseY() >= squarePos.y && game->GetMouseY() <= squarePos.y + GRID_SIZE.y) {
				if (game->GetKey(W).bPressed) {
					t.wallN = !t.wallN;
					if (y > 0) {
						Tile& neighbor = map[y - 1][x];
						neighbor.wallS = t.wallN;
					}
				}
				if (game->GetKey(D).bPressed) {
					t.wallE = !t.wallE;
					if (x < MAP_SIZE.x-1) {
						Tile& neighbor = map[y][x+1];
						neighbor.wallW = t.wallE;
					}
				}
				if (game->GetKey(S).bPressed) {
					t.wallS = !t.wallS;
					if (y < MAP_SIZE.y - 1) {
						Tile& neighbor = map[y+1][x];
						neighbor.wallN = t.wallS;
					}
				}
				if (game->GetKey(A).bPressed) {
					t.wallW = !t.wallW;
					if (x > 0) {
						Tile& neighbor = map[y][x-1];
						neighbor.wallE = t.wallW;
					}
				}
				if (game->GetMouse(0).bPressed) {
					t.enemyId = selectedEnemy;
				}
				if (game->GetMouse(1).bPressed) {
					t.enemyId = EnemyID::NONE;
				}
				game->FillRectDecal(squarePos, GRID_SIZE, { 0,0,255,64 });
			}
			//game->DrawRectDecal(squarePos,GRID_SIZE,{64,64,64});
			if (t.enemyId != EnemyID::NONE) {
				if (t.enemyId >= COIN) {
					game->DrawPolygonDecal(nullptr, {
							vf2d{0,(float) GRID_SIZE.y / 2} + squarePos ,
							vf2d{(float)GRID_SIZE.x / 2,0} + squarePos ,
							vf2d{(float)GRID_SIZE.x,(float)GRID_SIZE.y / 2} + squarePos ,
							vf2d{(float)GRID_SIZE.x / 2,(float)GRID_SIZE.y} + squarePos ,
						}, { {0,0},{0,0} ,{0,0} ,{0,0} }, game->GetData(t.enemyId).col);
				}
				else {
					game->DrawDecal(squarePos, game->circle, { 1,1 }, game->GetData(t.enemyId).col);
				}
			}
			game->DrawLineDecal(squarePos,{squarePos.x+ (float)GRID_SIZE.x,squarePos.y},t.wallN?WHITE:Pixel{64,64,64});
			game->DrawLineDecal({ squarePos.x + (float)GRID_SIZE.x,squarePos.y }, { squarePos.x + (float)GRID_SIZE.x,squarePos.y + (float)GRID_SIZE.y }, t.wallE ? WHITE : Pixel{ 64,64,64 });
			game->DrawLineDecal({ squarePos.x + (float)GRID_SIZE.x,squarePos.y + (float)GRID_SIZE.y }, { squarePos.x,squarePos.y + (float)GRID_SIZE.y }, t.wallS ? WHITE : Pixel{ 64,64,64 });
			game->DrawLineDecal({ squarePos.x,squarePos.y + (float)GRID_SIZE.y }, squarePos, t.wallW ? WHITE : Pixel{ 64,64,64 });
		}
	}
	game->DrawStringDecal({ 2,16 }, "Selected Enemy ("+std::to_string(selectedEnemy) + "): " + game->GetData(selectedEnemy).name, WHITE, {3,3});
	LoadLevelHandling();
}

void Editor::LoadLevelHandling() {
	if (!game->IsTextEntryEnabled()) {
		if (game->GetKey(F1).bPressed || reEnableTextEntry) {
			game->TextEntryEnable(true);
			reEnableTextEntry = false;
		}
	}
	else {
		game->FillRectDecal({ 0,0 }, { (float)game->ScreenWidth(),32 }, BLACK);
		game->DrawStringDecal({ 0,0 }, "Enter Level (1-30): " + game->TextEntryGetString(), WHITE, { 4,4 });
	}
}

void Editor::OnTextEntryComplete(const std::string& sText) {
	std::stringstream ss(game->TextEntryGetString());
	int level;
	ss >> level;
	if (level >= 1 && level <= 30) {
		game->TextEntryEnable(false);
		this->level = level;
		LoadLevel();
	}
	else {
		reEnableTextEntry = true;
	}
}

void Editor::LoadLevel() {
	std::ifstream file("assets/map/map"+std::to_string(level)+".map");
	if (file.good()) {
		while (file.good()) {
			std::string line;
			std::getline(file, line);
			int width = std::atoi(line.c_str());
			std::getline(file, line);
			int height = std::atoi(line.c_str());
			MAP_SIZE = { width,height };
			map.clear();
			for (int y = 0; y < MAP_SIZE.y; y++) {
				std::vector<Tile>row;
				for (int x = 0; x < MAP_SIZE.x; x++) {
					std::getline(file, line);
					int map_data = std::atoi(line.c_str());
					Tile newTile;
					newTile.blink = (map_data & 0b1000000000000000) >> 15;
					newTile.facingDir = FacingDirection((map_data & 0b0110000000000000) >> 13);
					newTile.enemyId = EnemyID((map_data & 0b0001111110000000) >> 7);
					newTile.wave1 = (map_data & 0b10000) >> 4;
					newTile.wave2 = (map_data & 0b100000) >> 5;
					newTile.wave3 = (map_data & 0b1000000) >> 6;
					newTile.wallN = (map_data & 0b1000) >> 3;
					newTile.wallE = (map_data & 0b100) >> 2;
					newTile.wallS = (map_data & 0b10) >> 1;
					newTile.wallW = (map_data & 0b1);
					row.push_back(newTile);
				}
				map.push_back(row);
			}
		}
	}
	else {
		std::cout << "Map Level " << level << " does not exist. Creating default map."<<std::endl;
		MAP_SIZE = { 5,5 };
		map.clear();
		for (int y = 0; y < MAP_SIZE.y; y++) {
			std::vector<Tile>row;
			for (int x = 0; x < MAP_SIZE.x; x++) {
				row.push_back({});
			}
			map.push_back(row);
		}
	}
}